#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "DnsAddressRecordForm.h"

#include <openwbem/OW_CIMValue.hpp>
#include <openwbem/OW_CIMQualifier.hpp>
#include <openwbem/OW_CIMException.hpp>
#include <openwbem/OW_Bool.hpp>
#include <iostream>

using namespace Gtk;
using namespace OpenWBEM;

DnsAddressRecordForm::DnsAddressRecordForm()
	: table(2, 2),
	name_label("Host Name"),
	address_label("IP Address")
{
	// set table parameters
	table.set_row_spacings(8);
	table.set_col_spacings(10);
	table.set_border_width(8);

	// set label properties
	name_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	name_label.show();
	address_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	address_label.show();
	
	// attach labels to the table
	table.attach(name_label, 0, 1, 0, 1, FILL, SHRINK);
	table.attach(address_label, 0, 1, 1, 2, FILL, SHRINK);

	// set entry properties
	name_entry.set_editable(false);
	name_entry.show();
	address_entry.show();

	// attach entries to the table
	table.attach(name_entry, 1, 2, 0, 1,
			FILL | EXPAND, SHRINK);
	table.attach(address_entry, 1, 2, 1, 2,
			FILL | EXPAND, SHRINK);
}

void
DnsAddressRecordForm::set_instance(const CIMObjectPath &cop,
		Reference<CIMInstance> inst)
{
	this->inst = inst;

	// fill in widgets with values from instance
	update();
}

Gtk::Widget *
DnsAddressRecordForm::getWidget()
{
	return &table;
}

void
DnsAddressRecordForm::update()
{
	OW_String string;
	OW_Bool boolean;
	
	// host name
	CIMValue value = inst->getProperty("OwnerName").getValue();
	if (value)
		value.get(string);
	else
		string = "";
	name_entry.set_text(string.c_str());
	
	// ip address
	value = inst->getProperty("IPAddress").getValue();
	if (value)
		value.get(string);
	else
		string = "";
	address_entry.set_text(string.c_str());
}

void
DnsAddressRecordForm::applyChanges()
{
	// address
	inst->setProperty("IPAddress",
			CIMValue(address_entry.get_text().c_str()));
}
