#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

// DNS plugin headers
#include "DnsPlugin.h"
#include "DnsAddressRecordForm.h"
#include "DnsMXRecordForm.h"
#include "DnsSOARecordForm.h"

// CIMBrowser headers
#include "ClassHandlerRegistry.h"
#include "ModifyInstanceDialog.h"
#include "InstancePropertyListForm.h"

using namespace OpenWBEM;
using namespace Gtk;

class DnsServerHandler : public ClassHandler
{
	virtual void do_properties(const CIMObjectPath &cop, Window *parent_window)
	{
		ModifyInstanceDialog *dlg = new ModifyInstanceDialog(cop);
		dlg->set_transient_for(*parent_window);
		dlg->add_form(new InstancePropertyListForm, "Advanced");
		dlg->show();
	}
	virtual String get_child_assoc_name()
	{
		return "Config4GNU_Bind_ServerZoneContainment";
	}
};

class DnsZoneHandler : public ClassHandler
{
	virtual void do_properties(const CIMObjectPath &cop, Window *parent_window)
	{
		ModifyInstanceDialog *dlg = new ModifyInstanceDialog(cop);
		dlg->set_transient_for(*parent_window);
		dlg->add_form(new InstancePropertyListForm, "Advanced");
		dlg->show();
	}
	virtual void activate(const CIMObjectPath &cop, MainWindow *main_window)
	{
		main_window->get_result_list()->remove_all_columns();
		main_window->get_result_list()->add_text_column("ElementName", "Name");
		main_window->get_result_list()->add_text_column("Type", "Type");
		main_window->get_result_list()->add_text_column("RecordData", "Data");
		main_window->get_result_list()->associators(cop,
			"Config4GNU_Bind_ZoneResourceRecordContainment");
	}
};

class DnsAddressHandler : public ClassHandler
{
	virtual void do_properties(const CIMObjectPath &cop, Window *parent_window)
	{
		ModifyInstanceDialog *dlg = new ModifyInstanceDialog(cop);
		dlg->set_transient_for(*parent_window);
		dlg->add_form(new DnsAddressRecordForm, "Address");
		dlg->add_form(new InstancePropertyListForm, "Advanced");
		dlg->show();
	}
};

class DnsMXHandler : public ClassHandler
{
	virtual void do_properties(const CIMObjectPath &cop, Window *parent_window)
	{
		ModifyInstanceDialog *dlg = new ModifyInstanceDialog(cop);
		dlg->set_transient_for(*parent_window);
		dlg->add_form(new DnsMXRecordForm, "Mail Exchanger");
		dlg->add_form(new InstancePropertyListForm, "Advanced");
		dlg->show();
	}
};

class DnsSOAHandler : public ClassHandler
{
	virtual void do_properties(const CIMObjectPath &cop, Window *parent_window)
	{
		CIMInstanceHandle objectref = CIMInstanceProxy::create(cop);
		ModifyInstanceDialog *dlg = new ModifyInstanceDialog(cop);
		dlg->set_transient_for(*parent_window);
		dlg->add_form(new DnsSOARecordForm(objectref),
			"Start of Authority (SOA)");
		dlg->add_form(new InstancePropertyListForm, "Advanced");
		dlg->show();
	}
};

extern "C" {
void
init_plugin()
{		
	ClassHandlerRegistry::add_handler("Config4GNU_Bind_DNSServer",
			new DnsServerHandler);
	ClassHandlerRegistry::add_handler("Config4GNU_Bind_DNSZone",
			new DnsZoneHandler);
	ClassHandlerRegistry::add_handler("Config4GNU_Bind_AType",
			new DnsAddressHandler);
	ClassHandlerRegistry::add_handler("Config4GNU_Bind_MXType",
			new DnsMXHandler);
	ClassHandlerRegistry::add_handler("Config4GNU_Bind_SOAType",
			new DnsSOAHandler);
}
} // end extern "C" block
