#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "NagiosHostDialog.h"
#include "CIMOMHandle.h"
#include "InstancePropertyListForm.h"

#include <gtkmm/messagedialog.h>
#include <gtkmm/stock.h>

#include <openwbem/OW_CIMValue.hpp>
#include <openwbem/OW_CIMQualifier.hpp>
#include <openwbem/OW_CIMException.hpp>
#include <openwbem/OW_CIMDateTime.hpp>
#include <openwbem/OW_Bool.hpp>
#include <openwbem/OW_DateTime.hpp>
#include <time.h>

using namespace Gtk;
using namespace OpenWBEM;

NagiosHostDialog::NagiosHostDialog(CIMObjectPath cop)
	: ModifyInstanceDialog(cop)
{
	set_title("Host Information");
	set_default_size(300, 300);

	// add forms
	add_form(new GeneralForm, "General");
	add_form(new ServicesForm, "Services");
	add_form(new InstancePropertyListForm, "Advanced");
	
	// make sure all children widgets are visible
	show_all_children();
}

NagiosHostDialog::NagiosHostDialog(CIMInstanceHandle host_handle)
	: ModifyInstanceDialog(host_handle->getObjectPath())
{
	set_title("Host Information");
	set_default_size(300, 300);

	// add forms
	add_form(new GeneralForm, "General");
	add_form(new ServicesForm, "Services");
	add_form(new InstancePropertyListForm, "Advanced");
	
	// make sure all children widgets are visible
	show_all_children();
}

NagiosHostDialog::GeneralForm::GeneralForm()
	: general_table(3, 2),
	general_status_label("Current Status:"),
	general_last_check_label("Last Check:"),
	general_output_label("Status Information:")
{
	general_vbox.pack_start(general_header_hbox, PACK_SHRINK);
	general_vbox.pack_start(general_separator, PACK_SHRINK);
	general_vbox.pack_start(general_table);
	general_vbox.set_border_width(6);
	general_vbox.set_spacing(4);

	general_header_hbox.pack_start(general_header_vbox);
	general_header_vbox.pack_start(general_host_name);
	general_header_vbox.pack_start(general_host_address);

	// set table parameters
	general_table.set_row_spacings(2);
	general_table.set_col_spacings(8);
	
	// set label alignments
	general_host_name.set_alignment(0.0, 0.5);
	general_host_address.set_alignment(0.0, 0.5);
	general_status_label.set_alignment(0.0, 0.5);
	general_status.set_alignment(0.0, 0.5);
	general_last_check_label.set_alignment(0.0, 0.5);
	general_last_check.set_alignment(0.0, 0.5);
	general_output_label.set_alignment(0.0, 0.5);
	
	// attach labels to the table
	general_table.attach(general_status_label, 0, 1, 0, 1, FILL, SHRINK);
	general_table.attach(general_last_check_label, 0, 1, 1, 2, FILL, SHRINK);

	// attach entries to the table
	general_table.attach(general_status, 1, 2, 0, 1,
			FILL | EXPAND, SHRINK);
	general_table.attach(general_last_check, 1, 2, 1, 2,
			FILL | EXPAND, SHRINK);
	
	// add plugin output to table
	general_table.attach(general_output_vbox, 0, 2, 2, 3);
	general_output_vbox.pack_start(general_output_label, PACK_SHRINK);
	general_output_scrolled.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC);
	general_output_scrolled.set_shadow_type(SHADOW_IN);
	general_output_scrolled.add(general_output_textview);
	general_output_vbox.pack_start(general_output_scrolled);
	general_output_textview.set_wrap_mode(WRAP_WORD);
	general_output_textview.set_editable(false);
}

void
NagiosHostDialog::GeneralForm::set_instance(const CIMObjectPath &cop,
		Reference<CIMInstance> inst)
{
	this->inst = inst;
	update();
}

Widget *
NagiosHostDialog::GeneralForm::getWidget()
{
	return &general_vbox;
}

void
NagiosHostDialog::GeneralForm::update()
{
	String string;
	
	// host name
	CIMValue value = inst->getProperty("ElementName").getValue();
	if (value)
		value.get(string);
	else
		string = "";
	general_host_name.set_text(string.c_str());
	
	// address
	value = inst->getProperty("Address").getValue();
	if (value)
		value.get(string);
	else
		string = "";
	general_host_address.set_text(string.c_str());
	
	// status
	value = inst->getProperty("Status").getValue();
	if (value)
		value.get(string);
	else
		string = "";
	general_status.set_text(string.c_str());

	// last update
	CIMDateTime x;
	value = inst->getProperty("LastCheckTime").getValue();
	if (value)
	{
		value.get(x);
		DateTime date = x.toDateTime();
		time_t lastCheckTime = date.get();
		char timebuf[200];
		strftime(timebuf, sizeof(timebuf), "%x %X",
				localtime(&lastCheckTime));
		string = timebuf;
	}
	else
		string = "";
	general_last_check.set_text(string.c_str());

	// plugin output
	value = inst->getProperty("PluginOutput").getValue();
	if (value)
		value.get(string);
	else
		string = "";
	general_output_textview.get_buffer()->set_text(string.c_str());
}

void
NagiosHostDialog::GeneralForm::applyChanges()
{
}

NagiosHostDialog::ServicesForm::ServicesForm()
	: services_list(false)
{
	services_list.show();
	services_scrolled.add(services_list);

	services_scrolled.set_border_width(6);
	services_scrolled.set_shadow_type(SHADOW_IN);
	services_scrolled.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC);
	services_scrolled.show();
}

void
NagiosHostDialog::ServicesForm::set_instance(const CIMObjectPath &cop,
		Reference<CIMInstance> inst)
{
	this->cop = cop;
	update();
}

Widget *
NagiosHostDialog::ServicesForm::getWidget()
{
	return &services_scrolled;
}

void
NagiosHostDialog::ServicesForm::update()
{
	// services
	services_list.show_host(cop);
}

void
NagiosHostDialog::ServicesForm::applyChanges()
{
}
