#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "NagiosHostList.h"
#include "CIMOMHandle.h"
#include "NagiosWindow.h"
#include "NagiosHostDialog.h"
#include "ModifyInstanceDialog.h"
#include "ClassHandlerRegistry.h"
#include "InstancePropertyListForm.h"

#include <openwbem/OW_Enumeration.hpp>
#include <openwbem/OW_CIMException.hpp>
#include <openwbem/OW_CIMProperty.hpp>
#include <openwbem/OW_CIMValue.hpp>
#include <openwbem/OW_CIMDateTime.hpp>
#include <gtkmm/cellrenderertext.h>
#include <time.h>
#include <openwbem/OW_DateTime.hpp>

using namespace Gtk;
using namespace OpenWBEM;

NagiosHostList::NagiosHostList()
{
	m_store = ListStore::create(Columns);
	set_model(m_store);

	append_column("Host", Columns.host);

	CellRendererText *renderer = manage(new CellRendererText());
	append_column("Status", *renderer);
	get_column(1)->add_attribute(renderer->property_text(), Columns.status);
	get_column(1)->add_attribute(renderer->property_background(),
			Columns.status_color);

	append_column("Last Check", Columns.last_check);
	append_column("Duration", Columns.duration);
	append_column("Status Information", Columns.information);
}

void
NagiosHostList::show_hostgroup(CIMObjectPath hostgroup_cop)
{
	m_store->clear();

	try
	{
		CIMInstanceEnumeration result;
		result = CIMOMHandle::associatorsE(hostgroup_cop);
	
		while (result.hasMoreElements())
		{
			CIMInstance inst = result.nextElement();
			Glib::ustring classname = inst.getClassName().c_str();

			TreeModel::iterator iter = m_store->append();
			TreeModel::Row row = *iter;
			
			//row[Columns.classname] = classname;
	
			String string;
			inst.getProperty("ElementName").getValue().get(string);
			row[Columns.host] = string.c_str();

			inst.getProperty("Status").getValue().get(string);
			row[Columns.status] = string.c_str();
			row[Columns.status_color] = (string == "UP" ? "green" :
					string == "DOWN" ? "red" : "gray");

			CIMDateTime x;
			inst.getProperty("LastCheckTime").getValue().get(x);
			time_t lastCheckTime = convertTime(x);
			char timebuf[200];
			strftime(timebuf, sizeof(timebuf), "%x %X",
					localtime(&lastCheckTime));
			row[Columns.last_check] = timebuf;

			inst.getProperty("LastStateChange").getValue().get(x);
			time_t lastStateChange = convertTime(x);
			row[Columns.duration] = duration2string(time(NULL)
					- lastStateChange);

			inst.getProperty("PluginOutput").getValue().get(string);
			row[Columns.information] = string.c_str();
			
			row[Columns.instobj] = inst;
		}
	}
	catch (CIMException e)
	{
		g_warning("%s", e.getMessage());
	}
}

time_t
NagiosHostList::convertTime(CIMDateTime x)
{
	DateTime date = x.toDateTime();
	return date.get();
}

Glib::ustring
NagiosHostList::duration2string(time_t duration)
{
	int seconds = duration % 60;
	duration /= 60;
	int minutes = duration % 60;
	duration /= 60;
	int hours = duration % 24;
	int days = duration / 24;

	char *tmp = g_strdup_printf("%dd %dh %dm %ds",
			days, hours, minutes, seconds);
	Glib::ustring result(tmp);
	g_free(tmp);
	return result;
}

void
NagiosHostList::on_row_activated(const TreeModel::Path &path,
		TreeViewColumn *column)
{
	TreeModel::iterator iter = m_store->get_iter(path);
	TreeModel::Row row = *iter;
	
	CIMInstance inst = row[Columns.instobj];
	on_instance_activated(inst);
}

void
NagiosHostList::on_instance_activated(CIMInstance inst)
{
	CIMObjectPath cop(CIMOMHandle::getDefaultNamespace(), inst);

	ClassHandler *handler = ClassHandlerRegistry::get_handler(
			inst.getClassName());
	if (handler != NULL)
	{
		handler->do_properties(cop, (Window *) get_toplevel());
	}
	else
	{
		g_message("classname = %s",
			inst.getClassName().c_str());
		ModifyInstanceDialog *dlg = new ModifyInstanceDialog(cop);
		dlg->add_form(new InstancePropertyListForm, "Advanced");
		dlg->show();
	}
}
