#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "NagiosWindow.h"
#include "CIMOMHandle.h"

#include <gtkmm.h>
#include <openwbem/OW_CIMProtocolIFC.hpp>
#include <openwbem/OW_HTTPClient.hpp>
#include <openwbem/OW_CIMXMLCIMOMHandle.hpp>
#include <openwbem/OW_CIMException.hpp>

using namespace Gtk;
using namespace OpenWBEM;

NagiosWindow::NagiosWindow(CIMObjectPath cop)
{
	set_title("Nagios");
	set_size_request(600, 450);
	
	// File menu:
	{
		Menu::MenuList &menulist = menu_file.items();
		
		menulist.push_back(Menu_Helpers::MenuElem("_Connect",
					SigC::slot(*this, &NagiosWindow::on_menu_file_connect)));
		menulist.push_back(Menu_Helpers::StockMenuElem(Stock::QUIT,
					SigC::slot(*this, &NagiosWindow::on_menu_file_quit)));
	}
	
	// View menu:
	{
		Menu::MenuList &menulist = menu_view.items();
		
		menulist.push_back(Menu_Helpers::MenuElem("_Find Class...",
				SigC::slot(*this, &NagiosWindow::on_menu_schema_find_class)));
	}

	// Help menu:
	{
		Menu::MenuList &menulist = menu_help.items();
		
		menulist.push_back(Menu_Helpers::MenuElem("_About Provider...",
				SigC::slot(*this, &NagiosWindow::on_menu_help_about)));
	}
	
	menu_bar.items().push_back(Menu_Helpers::MenuElem("_File", menu_file));
	menu_bar.items().push_back(Menu_Helpers::MenuElem("_View", menu_view));
	menu_bar.items().push_back(Menu_Helpers::MenuElem("_Help", menu_help));
	
	vbox.pack_start(menu_bar, PACK_SHRINK);
	
	vbox.pack_start(hpane);

	hpane.add(hostgroups_scrolled);
	hpane.add(services_scrolled);
	hpane.set_position(200);

	hostgroups_scrolled.set_shadow_type(SHADOW_IN);
	hostgroups_scrolled.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC);
	hostgroups_scrolled.add(hostgroups_list);
	services_scrolled.set_shadow_type(SHADOW_IN);
	services_scrolled.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC);
	services_scrolled.add(services_view);
	
	add(vbox);
	show_all_children();

	hostgroups_list.add_text_column("ElementName", "Host Group");
	hostgroups_list.enum_instances("Config4GNU_NagiosHostGroup");

	hostgroups_list.use_default_instance_activated_handler = false;
	hostgroups_list.signal_activated.connect(
			SigC::slot(*this, &NagiosWindow::on_hostgroup_activated));
}

void
NagiosWindow::on_menu_file_connect()
{
}

void
NagiosWindow::on_menu_file_quit()
{
	hide();
}

void
NagiosWindow::on_menu_schema_find_class()
{
}

void
NagiosWindow::on_menu_help_about()
{
}

void
NagiosWindow::on_hostgroup_activated(CIMObjectPath hostgroup_cop)
{
	services_view.show_hostgroup(hostgroup_cop);
}
