// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef BOUNDENTRY_H
#define BOUNDENTRY_H

#include "CIMInstanceProxy.h"
#include <gtkmm/entry.h>
#include <gtkmm/label.h>

/**
 * Consists of a Entry and Label tied to a specific property of a
 * specific CIM instance. The BoundEntry object will automatically set
 * the label to the property's "DisplayName" qualifier or the property
 * name itself if that qualifier is not defined. The entry will be
 * filled in with the property's current value.
 */
class BoundEntry
{
private:
	// PROPERTIES
	OpenWBEM::String property_name;
	Gtk::Entry *entry;
	Gtk::Label *label;
	CIMInstanceHandle objectref;

public:
	// CONSTRUCTOR/DESTRUCTOR
	BoundEntry(const OpenWBEM::String &property_name);
	~BoundEntry();

	/**
	 * Return this BoundEntry's Entry widget. The entry is not actually
	 * created until the first time this method is called.
	 */
	Gtk::Entry *get_entry()
	{
		if (entry == NULL)
			create_entry();
		return entry;
	}

	/**
	 * Return this BoundEntry's Label widget. The label is not actually
	 * created until the first time this method is called. If you want
	 * to customize the label text, simply do not access the BoundEntry's
	 * label widget and instead use your own label.
	 */
	Gtk::Label *get_label()
	{
		if (label == NULL)
			create_label();
		return label;
	}

	/**
	 * Gets the property's display name. This value is used when creating
	 * the BoundEntry's label. It is the value of the property's
	 * "DisplayName" qualifier, or the property name if the qualifier is
	 * not defined.
	 */	
	Glib::ustring get_property_display_name();
	
	// MUTATORS
	void set_inst(const CIMInstanceHandle &objectref);

	// OTHER METHODS
	void create_entry();
	void create_label();
	void update();
	void applyChanges();
};

#endif /* BOUNDENTRY_H */
