// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef CIMINSTANCEPROXY_H
#define CIMINSTANCEPROXY_H

#include <openwbem/OW_Reference.hpp>
#include <openwbem/OW_CIMObjectPath.hpp>
#include <openwbem/OW_CIMInstance.hpp>
#include <openwbem/OW_CIMClass.hpp>

class CIMInstanceProxy;
typedef OpenWBEM::Reference<CIMInstanceProxy> CIMInstanceHandle;

/**
 * Represents a particular instance's data. This class is new, so there
 * are many places that should use this class but don't yet because
 * they haven't been converted. Instances of this class should always
 * be accessed using a Reference object, which tracks how many places
 * are using the instance and when the instance data can be freed.
 */
class CIMInstanceProxy
{
private:
	// PRIVATE CONSTRUCTOR/DESTRUCTOR
	CIMInstanceProxy(const OpenWBEM::CIMObjectPath &cop);

protected:
	// PROPERTIES
	OpenWBEM::CIMObjectPath cop;
	bool fetched_instance;
	OpenWBEM::CIMInstance inst;

	CIMInstanceProxy(const OpenWBEM::String &ns,
		const OpenWBEM::CIMInstance &inst);

	void fetch_instance();

public:
	/**
	 * Creates a new instance and returns a reference to it.
	 * New object handles must be created using this static method,
	 * which ensures that only object handle references are being
	 * passed around.
	 */
	static CIMInstanceHandle create(const OpenWBEM::CIMObjectPath &cop);

	static CIMInstanceHandle new_instance(const OpenWBEM::String &ns,
		const OpenWBEM::String &className);

	// methods to get/set property data
	OpenWBEM::CIMPropertyArray getProperties();
	OpenWBEM::CIMProperty getProperty(const OpenWBEM::String &propertyName);
	CIMInstanceHandle getRefProperty(const OpenWBEM::String &propertyName);

	void setProperty(const OpenWBEM::String &propertyName,
		const OpenWBEM::CIMValue &value);
	void setRefProperty(const OpenWBEM::String &propertyName,
		CIMInstanceHandle value);

	virtual void commitChanges();
	virtual void destroy();

	// methods to get related instances
	std::vector<CIMInstanceHandle> references(
		const OpenWBEM::String &resultClass = "");

	// other methods
	OpenWBEM::String toString() const;

	OpenWBEM::CIMObjectPath getObjectPath() const
	{
		return cop;
	}
	bool equals(CIMInstanceHandle rhs) const
	{
		return getObjectPath().equals(rhs->getObjectPath());
	}
	
	OpenWBEM::String getNameSpace() const
	{
		return cop.getNameSpace();
	}

	virtual ~CIMInstanceProxy();
};

#endif /* CIMINSTANCEPROXY_H */
