// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "CIMOMHandle.h"
#include "ConnectDialog.h"
#include <openwbem/OW_CIMProtocolIFC.hpp>
#include <openwbem/OW_HTTPClient.hpp>
#include <openwbem/OW_CIMXMLCIMOMHandle.hpp>
#include <openwbem/OW_CIMObjectPath.hpp>
#include <openwbem/OW_CIMClass.hpp>
#include <iostream>
#include <assert.h>

using namespace OpenWBEM;
using namespace Gtk;

ClientAuthCBIFCRef CIMOMHandle::getLoginInfo;
String CIMOMHandle::defaultNamespace;
CIMOMHandleIFCRef CIMOMHandle::defaultHandle;

class GetLoginInfo : public ClientAuthCBIFC
{
public:
	String name;
	String password;

	bool getCredentials(const String& realm, String& name,
				String& passwd, const String& details)
	{
		if (this->name != "")
		{
			name = this->name;
			passwd = this->password;
			
			// once we return the username/password, blank ours out so
			// if it failed we know to prompt the user
			//this->name = "";
			//this->password = "";
			
			return true;
		}
		// TODO - show a dialog box
		return false;
	}
};

void
CIMOMHandle::initDefaultHandle()
{
	ConnectDialog dialog;
	if (dialog.run() != RESPONSE_OK)
	{
		// TODO: throw exception... user canceled connect exception
		assert(0);
	}
	dialog.hide();
	
	String url;
	String username;
	String password;
	dialog.getParams(url, defaultNamespace, username, password);
	
	GetLoginInfo *pLoginInfo = new GetLoginInfo;
	pLoginInfo->name = username;
	pLoginInfo->password = password;	
	getLoginInfo = ClientAuthCBIFCRef(pLoginInfo);

	CIMProtocolIFCRef client(new HTTPClient(url));
	client->setLoginCallBack(getLoginInfo);

	defaultHandle = new CIMXMLCIMOMHandle(client);
}

CIMInstanceEnumeration
CIMOMHandle::enumInstancesE(const String &className)
{
	if (defaultHandle == NULL)
	{
		initDefaultHandle();
	}
	assert(defaultHandle != NULL);
	return defaultHandle->enumInstancesE(defaultNamespace, className);
}

std::vector<CIMInstanceHandle>
CIMOMHandle::getAllInstances(const String &className)
{
	if (defaultHandle == NULL)
	{
		initDefaultHandle();
	}
	assert(defaultHandle != NULL);
	
	CIMObjectPathEnumeration copEnum = defaultHandle->enumInstanceNamesE(
			defaultNamespace, className);

	std::vector<CIMInstanceHandle> result;
	while (copEnum.hasMoreElements())
	{
		result.push_back(CIMInstanceProxy::create(copEnum.nextElement()));
	}

	return result;
}

CIMClassEnumeration
CIMOMHandle::enumClassE(const String &className)
{
	if (defaultHandle == NULL)
	{
		initDefaultHandle();
	}
	assert(defaultHandle != NULL);
	return defaultHandle->enumClassE(defaultNamespace, className);
}

CIMClassEnumeration
CIMOMHandle::referencesClassesE(const CIMObjectPath &className)
{
	if (defaultHandle == NULL)
	{
		initDefaultHandle();
	}
	assert(defaultHandle != NULL);

	return defaultHandle->referencesClassesE(defaultNamespace, className,
				"", "", WBEMFlags::E_INCLUDE_QUALIFIERS,
				WBEMFlags::E_INCLUDE_CLASS_ORIGIN);
}

CIMClass
CIMOMHandle::getClass(const String &className)
{
	if (defaultHandle == NULL)
	{
		initDefaultHandle();
	}
	assert(defaultHandle != NULL);

	return defaultHandle->getClass(defaultNamespace, className);
}

CIMInstanceEnumeration
CIMOMHandle::associatorsE(const CIMObjectPath &refObject,
		const String &assocClass)
{
	if (defaultHandle == NULL)
	{
		initDefaultHandle();
	}
	assert(defaultHandle != NULL);

	return defaultHandle->associatorsE(defaultNamespace, refObject, assocClass);
}

CIMInstanceEnumeration
CIMOMHandle::referencesE(const CIMObjectPath &refObject,
		const String &assocClass)
{
	if (defaultHandle == NULL)
	{
		initDefaultHandle();
	}
	assert(defaultHandle != NULL);

	return defaultHandle->referencesE(defaultNamespace, refObject, assocClass);
}

CIMInstance
CIMOMHandle::getInstance(const CIMObjectPath &cop)
{
	if (defaultHandle == NULL)
	{
		initDefaultHandle();
	}
	assert(defaultHandle != NULL);

//	std::cout << "Getting instance:" << std::endl;
//	std::cout << cop.toString() << std::endl;
//	std::cout << "  Host: " << cop.getHost() << std::endl;
//	std::cout << "  Namespace: " << cop.getNameSpace() << std::endl;
//	std::cout << "default namespace: " << defaultNamespace << std::endl;
	
	return defaultHandle->getInstance(defaultNamespace, cop,
					WBEMFlags::E_NOT_LOCAL_ONLY,
					WBEMFlags::E_INCLUDE_QUALIFIERS);
}

void
CIMOMHandle::modifyInstance(const CIMInstance &inst)
{
	if (defaultHandle == NULL)
	{
		initDefaultHandle();
	}
	assert(defaultHandle != NULL);

	defaultHandle->modifyInstance(defaultNamespace, inst);
}

CIMObjectPathEnumeration
CIMOMHandle::associatorNamesE(const CIMObjectPath &cop,
		const String &assocClass)
{
	if (defaultHandle == NULL)
	{
		initDefaultHandle();
	}
	assert(defaultHandle != NULL);

	return defaultHandle->associatorNamesE(defaultNamespace, cop, assocClass);
}

CIMObjectPathEnumeration
CIMOMHandle::referenceNamesE(const CIMObjectPath &cop, const String &resultClass)
{
	if (defaultHandle == NULL)
	{
		initDefaultHandle();
	}
	assert(defaultHandle != NULL);

	return defaultHandle->referenceNamesE(cop.getNameSpace(), cop, resultClass);
}

void
CIMOMHandle::createInstance(const CIMInstance &inst)
{
	if (defaultHandle == NULL)
	{
		initDefaultHandle();
	}
	assert(defaultHandle != NULL);

	defaultHandle->createInstance(defaultNamespace, inst);
}

void
CIMOMHandle::deleteInstance(const CIMObjectPath &cop)
{
	if (defaultHandle == NULL)
	{
		initDefaultHandle();
	}
	assert(defaultHandle != NULL);
	
	defaultHandle->deleteInstance(defaultNamespace, cop);
}
