// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef CIMOMHANDLE_H
#define CIMOMHANDLE_H

#include <openwbem/OW_String.hpp>
#include <openwbem/OW_ClientAuthCBIFC.hpp>
#include <openwbem/OW_CIMOMHandleIFC.hpp>
#include "CIMInstanceProxy.h"
#include <vector>

/**
 * Container for a number of static methods for communicating with the
 * CIMOM. Here you will find methods for getting a particular instance's
 * data, or enumerating instances or classes.
 */
class CIMOMHandle
{
private:
	static OpenWBEM::ClientAuthCBIFCRef getLoginInfo;
	static OpenWBEM::String defaultNamespace;
	static OpenWBEM::CIMOMHandleIFCRef defaultHandle;

public:
	static void initDefaultHandle();

	static OpenWBEM::CIMInstanceEnumeration enumInstancesE(
			const OpenWBEM::String &className);
	static std::vector<CIMInstanceHandle> getAllInstances(
			const OpenWBEM::String &className);

	static OpenWBEM::CIMClassEnumeration enumClassE(
			const OpenWBEM::String &className);
	static OpenWBEM::CIMClassEnumeration referencesClassesE(
			const OpenWBEM::CIMObjectPath &className);
	static OpenWBEM::CIMClass getClass(const OpenWBEM::String &className);
	static OpenWBEM::CIMInstanceEnumeration associatorsE(
			const OpenWBEM::CIMObjectPath &refObject,
			const OpenWBEM::String &assocClass = "");
	static OpenWBEM::CIMInstanceEnumeration referencesE(
			const OpenWBEM::CIMObjectPath &refObject,
			const OpenWBEM::String &assocClass = "");
	static OpenWBEM::CIMInstance getInstance(
			const OpenWBEM::CIMObjectPath &cop);
	static void modifyInstance(const OpenWBEM::CIMInstance &inst);
	static OpenWBEM::CIMObjectPathEnumeration associatorNamesE(
			const OpenWBEM::CIMObjectPath &cop,
			const OpenWBEM::String &assocClass = "");
			
	static OpenWBEM::CIMObjectPathEnumeration referenceNamesE(
			const OpenWBEM::CIMObjectPath &cop,
			const OpenWBEM::String &resultClass = "");

	static void createInstance(const OpenWBEM::CIMInstance &inst);
	static void deleteInstance(const OpenWBEM::CIMObjectPath &cop);

	static OpenWBEM::String getDefaultNamespace()
		{ return defaultNamespace; }
};

#endif /* CIMOMHANDLE_H */
