// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef CLASSHANDLER_H
#define CLASSHANDLER_H

#include <openwbem/OW_CIMObjectPath.hpp>
#include <gtkmm/window.h>
#include "MainWindow.h"
#include "DefaultInstanceScopeItem.h"

/**
 * Defines how to handle instances of a particular CIM class.
 */
class ClassHandler
{
public:
	/**
	 * Perform the default activation implementation for objects of
	 * this class.
	 */
	virtual void activate(const OpenWBEM::CIMObjectPath &cop,
		MainWindow *main_window)
	{
	}
	
	/**
	 * Perform the default action for objects of this class.
	 * This is called whenever an item is double-clicked in the result
	 * pane.
	 */
	virtual void do_default_action(const OpenWBEM::CIMObjectPath &cop,
		MainWindow *main_window)
	{
		do_properties(cop, main_window);
	}
	
	virtual void do_default_action(CIMInstanceHandle inst,
		MainWindow *main_window)
	{
		do_properties(inst, main_window);
	}
	
	/**
	 * Display a properties window for an object of this class.
	 */
	virtual void do_properties(const OpenWBEM::CIMObjectPath &cop,
		Gtk::Window *parent_window) = 0;
	
	virtual void do_properties(CIMInstanceHandle inst,
		Gtk::Window *parent_window)
	{
		do_properties(inst->getObjectPath(), parent_window);
	}
	
	/**
	 * Get the association class name used to identify child scope
	 * items.
	 */
	virtual OpenWBEM::String get_child_assoc_name()
	{
		return "";
	}
	
	/**
	 * Create a scope item for an object of this class.
	 */
	virtual ScopeItemRef create_scope_item(
			const OpenWBEM::CIMObjectPath &cop,
			MainWindow *main_window)
	{
		return ScopeItemRef(new DefaultInstanceScopeItem(cop, main_window));
	}
};

#endif /* CLASSHANDLER_H */
