// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "ClassHandlerRegistry.h"
#include "PropertyPagesDialog.h"
#include "InstancePropertyListForm.h"

using namespace OpenWBEM;
using namespace Gtk;

std::map<String, ClassHandler *> ClassHandlerRegistry::class_handlers;
ClassHandlerRegistry::DefaultClassHandler ClassHandlerRegistry::defaultClassHandler;

void
ClassHandlerRegistry::add_handler(String className, ClassHandler *handler)
{
	String lowercaseClassName(className);
	lowercaseClassName.toLowerCase();
	class_handlers[lowercaseClassName] = handler;
}

ClassHandler *
ClassHandlerRegistry::get_handler(const String &className)
{
	String lowercaseClassName(className);
	lowercaseClassName.toLowerCase();
	return class_handlers.count(lowercaseClassName) == 1 ?
		class_handlers[lowercaseClassName] : get_default_handler();
}

void
ClassHandlerRegistry::DefaultClassHandler::do_properties(
		const OpenWBEM::CIMObjectPath &cop, Window *parent_window)
{
	g_warning("deprecated");
}

void
ClassHandlerRegistry::DefaultClassHandler::do_properties(
		CIMInstanceHandle inst, Window *parent_window)
{
	PropertyPagesDialog *dlg = new PropertyPagesDialog;
	dlg->set_title("Modify Instance");
	dlg->set_transient_for(*parent_window);
	dlg->add_form(new InstancePropertyListForm(inst),
		"Properties");
	dlg->show();
}
