// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef CLASSHANDLERREGISTRY_H
#define CLASSHANDLERREGISTRY_H

#include "ClassHandler.h"
#include <openwbem/OW_String.hpp>
#include <map>

/**
 * A registry for CIM class names and their class handlers.
 * When a plugin initializes, it registers one or more class handlers.
 * Then, when instances of those classes are seen, the plugin's code
 * is executed to handle those instances.
 */
class ClassHandlerRegistry
{
private:
	class DefaultClassHandler : public ClassHandler
	{
		virtual void do_properties(const OpenWBEM::CIMObjectPath &cop,
			Gtk::Window *parent_window);
		virtual void do_properties(CIMInstanceHandle inst,
			Gtk::Window *parent_window);
	};

	static std::map<OpenWBEM::String, ClassHandler *> class_handlers;
	static DefaultClassHandler defaultClassHandler;

public:
	/**
	 * Add a CIM class and its handler to the registry.
	 */
	static void add_handler(OpenWBEM::String className, ClassHandler *handler);

	/**
	 * Return a pointer to a class's handler.
	 */
	static ClassHandler *get_handler(const OpenWBEM::String &className);

	static ClassHandler *get_default_handler()
	{
		return &defaultClassHandler;
	}
};

#endif /* CLASSHANDLERREGISTRY_H */
