// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "ClassList.h"

#include <openwbem/OW_CIMProtocolIFC.hpp>
#include <openwbem/OW_HTTPClient.hpp>
#include <openwbem/OW_CIMXMLCIMOMHandle.hpp>
#include <openwbem/OW_Enumeration.hpp>
#include "ClassInformationDialog.h"
#include <openwbem/OW_CIMException.hpp>
#include "CIMOMHandle.h"

using namespace Gtk;
using namespace Glib;
using namespace OpenWBEM;

ClassList::ClassList()
{
	m_store = TreeStore::create(m_Columns);
	set_model(m_store);
	append_column("Class Name", m_Columns.classname);
}

void
ClassList::set_base_class(String className)
{
	m_store->clear();

	try
	{
		CIMClassEnumeration result;
		result = CIMOMHandle::enumClassE(className);
		while (result.hasMoreElements())
		{
			CIMClass klass = result.nextElement();
			ustring classname = klass.getName().c_str();
			
			TreeModel::iterator iter = m_store->append();
			TreeModel::Row row = *iter;
			
			row[m_Columns.classname] = classname;
			row[m_Columns.classobj] = klass;
			
			expand_subclasses(klass, row);
		}
	}
	catch (CIMException e)
	{
		g_warning("%s", e.getMessage());
	}
}

void
ClassList::expand_subclasses(CIMClass klass, TreeModel::Row parent_row)
{
	CIMClassEnumeration result;
	result = CIMOMHandle::enumClassE(klass.getName());
	while (result.hasMoreElements())
	{
		CIMClass subclass = result.nextElement();
		ustring classname = subclass.getName().c_str();
		
		TreeModel::iterator iter = m_store->append(parent_row.children());
		TreeModel::Row row = *iter;
		
		row[m_Columns.classname] = classname;
		row[m_Columns.classobj] = subclass;
		
		expand_subclasses(subclass, row);
	}	
}

void
ClassList::on_row_activated(const TreeModel::Path &path,
		TreeViewColumn *column)
{
	TreeModel::iterator iter = m_store->get_iter(path);
	TreeModel::Row row = *iter;
	
	CIMClass class_object = row[m_Columns.classobj];
	String classname = class_object.getName();

	ClassInformationDialog *dialog = new ClassInformationDialog(class_object);
	dialog->set_transient_for(*(Dialog *) get_toplevel());
	dialog->show();
}
