// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef CLASSLIST_H
#define CLASSLIST_H

#include <gtkmm/treeview.h>
#include <gtkmm/treestore.h>
#include <openwbem/OW_CIMOMHandleIFC.hpp>
#include <openwbem/OW_CIMClass.hpp>

/**
 * A widget that displays a list of classes and subclasses.
 */
class ClassList : public Gtk::TreeView
{
public:
	/**
	 * Create a new class list widget.
	 */
	ClassList();

	/**
	 * Set the name of the base class that the class widget displays.
	 */
	virtual void set_base_class(OpenWBEM::String className);

protected:
	/**
	 * Recursively list a class's subclasses, adding them to the tree store.
	 */
	void expand_subclasses(OpenWBEM::CIMClass klass,
			Gtk::TreeModel::Row parent_row);

	class ModelColumns : public Gtk::TreeModel::ColumnRecord
	{
	public:
		ModelColumns()
		{
			add(classname);
			add(classobj);
		}
		Gtk::TreeModelColumn<Glib::ustring> classname;
		Gtk::TreeModelColumn<OpenWBEM::CIMClass> classobj;
	};
	
	// PROPERTIES
	ModelColumns m_Columns;
	Glib::RefPtr<Gtk::TreeStore> m_store;

	// SIGNAL HANDLERS
	virtual void on_row_activated(const Gtk::TreeModel::Path &path,
			Gtk::TreeViewColumn *column);
};

#endif /* CLASSLIST_H */
