// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef EDITPROPERTYDIALOG_H
#define EDITPROPERTYDIALOG_H

#include <gtkmm/dialog.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/textview.h>
#include <gtkmm/combo.h>
#include <openwbem/OW_CIMProperty.hpp>

/**
 * A dialog for editing a single CIM property. The dialog theoretically
 * chooses an appropriate widget for editing the property's value,
 * based solely on the information expressed in the CIM schema.
 * The dialog will also display the property's description as found
 * in the CIM schema if one is available.
 */
class EditPropertyDialog : public Gtk::Dialog
{
protected:
	/// A copy of the property that's being edited.
	OpenWBEM::CIMProperty property;

	/// Holds the widgets of the dialog.
	Gtk::VBox vbox;
	/// Allows description to scroll if it's too long to fit in the dialog.
	/// This widget (and the text view it holds) is hidden if no
	/// description is available.
	Gtk::ScrolledWindow description_scrolled;
	/// Read-only text view that displays the property's description.
	Gtk::TextView description_view;

	/// A label saying "Description:"
	Gtk::Label description_label;
	/// A label saying "Value:"
	Gtk::Label value_label;

protected:
	/**
	 * Create a new EditPropertyDialog. Only subclasses can
	 * call this constructor. Otherwise, use the "create" method.
	 */
	EditPropertyDialog(const OpenWBEM::CIMProperty &property);

	/**
	 * Return as a string whatever text the user has entered
	 * in for a value.
	 */
	virtual OpenWBEM::String getStringValue() const = 0;
	/**
	 * Read the "Description" qualifier from the property object
	 * and display its contents in the description view.
	 */
	void update_description();

public:
	/**
	 * Create a new EditPropertyDialog, using an appropriate subclass
	 * of EditPropertyDialog depending on what type of property is
	 * given.
	 */
	static EditPropertyDialog *create(const OpenWBEM::CIMProperty &property);
	virtual ~EditPropertyDialog();

	/**
	 * Generate a new CIMValue object for the new property value.
	 */
	virtual OpenWBEM::CIMValue getValue() const;

protected:
	// DIALOG HANDLERS

	// SIGNAL HANDLERS
};

#endif /* EDITPROPERTYDIALOG_H */
