// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef FORM_H
#define FORM_H

#include <gtkmm/widget.h>
#include <openwbem/OW_Reference.hpp>
#include <openwbem/OW_CIMInstance.hpp>
#include <openwbem/OW_CIMOMHandleIFC.hpp>

/**
 * A bunch of widgets for manipulating a particular CIM instance.
 * The ModifyInstanceDialog displays one or more instances derived
 * from the Form class.
 */
class Form
{
public:
	virtual ~Form();

	/**
	 * Sets the instance that this form is to manipulate.
	 * This is called immediately after the form is created, which
	 * is awkward, so this method may be going away soon.
	 */
	virtual void set_instance(const OpenWBEM::CIMObjectPath &cop,
			OpenWBEM::Reference<OpenWBEM::CIMInstance> inst) = 0;

	/**
	 * Accesses the top-level widget of the form, usually some sort
	 * of container which itself holds widgets.
	 */
	virtual Gtk::Widget *getWidget() = 0;

	/**
	 * Updates the underlying CIM instance. The instance data should be
	 * updated to reflect the values that have been entered into the widgets on
	 * the form.
	 */
	virtual void applyChanges() = 0;
};

#endif /* FORM_H */
