// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef INSTANCEASSOCIATIONSFORM_H
#define INSTANCEASSOCIATIONSFORM_H

#include <gtkmm/scrolledwindow.h>
#include <gtkmm/treeview.h>
#include <gtkmm/menu.h>

#include "Form.h"
#include "CIMInstanceProxy.h"
#include "InstanceList.h"

class InstanceAssociationsList : public Gtk::TreeView
{
private:
	class MyModelColumns : public Gtk::TreeModel::ColumnRecord
	{
	public:
		MyModelColumns()
		{
			add(assocClassName);
			add(propertyName);
			add(relation);
			add(instanceName);
			add(referenceHandle);
			add(instanceHandle);
		}
		Gtk::TreeModelColumn<Glib::ustring> assocClassName;
		Gtk::TreeModelColumn<Glib::ustring> propertyName;
		Gtk::TreeModelColumn<Glib::ustring> relation;
		Gtk::TreeModelColumn<Glib::ustring> instanceName;
		Gtk::TreeModelColumn<CIMInstanceHandle> referenceHandle;
		Gtk::TreeModelColumn<CIMInstanceHandle> instanceHandle;
	};
	MyModelColumns Columns;
	Gtk::Menu context_menu;

	CIMInstanceHandle baseHandle;
	void add_reference(const CIMInstanceHandle &refInst);

public:
	InstanceAssociationsList();

	void setBaseInstance(CIMInstanceHandle baseHandle);

protected:
	virtual bool on_button_press_event(GdkEventButton *event);
	void on_menu_new_association();
	void on_menu_delete_association();
};	

/**
 * A form for viewing associations of a given CIM instance. This form
 * presents a scrollable list of instances that the current instance is
 * associated with.
 */
class InstanceAssociationsForm : public Form
{
private:
	/**
	 * A handle to the CIM instance this form is representing.
	 */
	CIMInstanceHandle objectref;

	InstanceAssociationsList instances_list;
	Gtk::ScrolledWindow instances_scrolled;

public:
	// CONSTRUCTOR
	InstanceAssociationsForm(CIMInstanceHandle objectref);

	// METHODS DEFINED IN Form
	virtual void set_instance(const OpenWBEM::CIMObjectPath &cop,
			OpenWBEM::Reference<OpenWBEM::CIMInstance> inst);
	virtual Gtk::Widget *getWidget();
	virtual void applyChanges();
};

#endif /* INSTANCEASSOCIATIONSFORM_H */
