// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "ModifyInstanceDialog.h"
#include "CIMOMHandle.h"

#include <gtkmm/messagedialog.h>
#include <gtkmm/stock.h>

#include <openwbem/OW_CIMValue.hpp>
#include <openwbem/OW_CIMQualifier.hpp>
#include <openwbem/OW_CIMException.hpp>
#include <openwbem/OW_Bool.hpp>
#include <iostream>

using namespace Gtk;
using namespace OpenWBEM;

ModifyInstanceDialog::ModifyInstanceDialog(CIMObjectPath cop)
{
	set_title("Modify Instance");
	set_default_size(300, 300);

	try
	{
		this->cop = cop;
		this->inst = Reference<CIMInstance>(new CIMInstance(
				CIMOMHandle::getInstance(cop)));
	}
	catch (CIMException e)
	{
		MessageDialog dialog(e.getMessage(), MESSAGE_ERROR);
		dialog.run();
		exit(1);
	}

	// add notebook to dialog
	get_vbox()->pack_start(notebook);

	// add buttons
	add_button(Stock::CANCEL, RESPONSE_CANCEL);
	add_button(Stock::APPLY, RESPONSE_APPLY);
	add_button(Stock::OK, RESPONSE_OK);
	set_default_response(RESPONSE_OK);
	
	// make sure all children widgets are visible
	show_all_children();
}

ModifyInstanceDialog::~ModifyInstanceDialog()
{
	for (unsigned int i = 0; i < forms.size(); i++)
	{
		delete forms[i];
	}
}

void
ModifyInstanceDialog::add_form(Form *form, Glib::ustring title)
{
	forms.push_back(form);
	form->set_instance(cop, inst);
	form->getWidget()->show();
	notebook.append_page(*form->getWidget(), title);
}

void
ModifyInstanceDialog::on_response(int response_id)
{
	if (response_id == RESPONSE_APPLY || response_id == RESPONSE_OK)
	{
		applyChanges();
	}

	if (response_id == RESPONSE_CANCEL || response_id == RESPONSE_OK)
		delete this;
}

void
ModifyInstanceDialog::applyChanges()
{
	for (unsigned int i = 0; i < forms.size(); i++)
	{
		forms[i]->applyChanges();
	}

	try
	{
		// call modify operation
		CIMOMHandle::modifyInstance(*inst);
	}
	catch (CIMException e)
	{
		MessageDialog dialog(*this, e.getMessage(), MESSAGE_ERROR);
		dialog.run();
	}
}
