// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef PROPERTYPAGESDIALOG_H
#define PROPERTYPAGESDIALOG_H

#include <gtkmm/dialog.h>
#include <gtkmm/notebook.h>
#include "Form.h"

/**
 * A dialog that displays one or more "forms" for editing a CIM
 * instance.
 */
class PropertyPagesDialog : public Gtk::Dialog
{
protected:
	/**
	 * The Notebook widget that holds the toplevel widgets of the
	 * Forms held by this dialog.
	 */
	Gtk::Notebook notebook;

	/**
	 * A list of pointers to the forms shown in this dialog.
	 */
	std::vector<Form *> forms;

public:
	// CONSTRUCTOR
	PropertyPagesDialog();
	virtual ~PropertyPagesDialog();

	/**
	 * Add a form to this dialog. The caller must identify what title
	 * gets displayed in the notebook tab.
	 */
	void add_form(Form *form, Glib::ustring title);

protected:
	// SIGNAL HANDLERS
	virtual void on_response(int response_id);

	/**
	 * Calls each form's applyChanges method. This method is called
	 * when OK or Apply is clicked.
	 */
	virtual void applyChanges();
};

#endif /* PROPERTYPAGESDIALOG_H */
