// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef SCOPEITEM_H
#define SCOPEITEM_H

#include <gtkmm/treemodel.h>
#include <gtkmm/menu.h>
#include <openwbem/OW_Reference.hpp>

class MainWindow;

/**
 * Represents an item in a scope pane. It is tied to a particular
 * instance of MainWindow. When a scope item is selected, queried for 
 * properties, or something like that, methods on this interface will be
 * called.
 */
class ScopeItem
{
public:
	virtual ~ScopeItem() {};
		
	/**
	 * Called when a scope item is selected. This method must be
	 * overridden. Typical behavior would be to show some items in the result
	 * pane.
	 */
	virtual void activate() = 0;

	/**
	 * Add child items to the scope pane for this scope item. This is
	 * called whenever a scope item is activated. Use the provided
	 * handle to the current scope item row to add child scope items.
	 */
	virtual void expand_row(Gtk::TreeModel::Row &row) = 0;

	/**
	 * Add menu items to the main window's "Action" window. This is
	 * called when a scope item is activated.
	 */
	virtual void init_actions_menu(Gtk::Menu &action_menu) { }
};
typedef OpenWBEM::Reference<ScopeItem> ScopeItemRef;

#endif /* SCOPEITEM_H */
