// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "SimpleCIMForm.h"

#include <openwbem/OW_CIMProperty.hpp>
#include <openwbem/OW_CIMValue.hpp>
#include <openwbem/OW_CIMQualifier.hpp>
#include <openwbem/OW_CIMException.hpp>
#include <openwbem/OW_Bool.hpp>

using namespace Gtk;
using namespace OpenWBEM;
using namespace std;

SimpleCIMForm::SimpleCIMForm(CIMInstanceHandle objectref)
	: objectref(objectref)
{
}

SimpleCIMForm::~SimpleCIMForm()
{
	for (vector<BoundEntry *>::iterator iter = bound_entries.begin();
		iter != bound_entries.end(); iter++)
	{
		BoundEntry *entry = *iter;
		delete entry;
	}
}

BoundEntry *
SimpleCIMForm::new_entry(const String &propertyName)
{
	BoundEntry *entry = new BoundEntry(propertyName);
	entry->get_label();
	entry->get_entry();
	entry->set_inst(objectref);
	entry->update();
	bound_entries.push_back(entry);
	return entry;
}

void
SimpleCIMForm::set_instance(const CIMObjectPath &cop,
		Reference<CIMInstance> inst)
{
}

void
SimpleCIMForm::update()
{
	for (vector<BoundEntry *>::iterator iter = bound_entries.begin();
		iter != bound_entries.end(); iter++)
	{
		BoundEntry *entry = *iter;
		entry->update();
	}
}

void
SimpleCIMForm::applyChanges()
{
	for (vector<BoundEntry *>::iterator iter = bound_entries.begin();
		iter != bound_entries.end(); iter++)
	{
		BoundEntry *entry = *iter;
		entry->applyChanges();
	}
}
